#mortgageGUI.py
"""Copyright 2010 Joshua Grigonis
This file is written by Joshua Grigonis.  
Redistribution or reuse is not permitted without express written consent.

contact: josh.grigonis@gmail.com
"""

# Note that this source file uses spaces, not tabs, as opposed to the others

import Tkinter as tk
import tkMessageBox

import convertTypes
import mortgage
import mortgageExceptions

#=============================================================================

GUIVERSION = 0.6

#=============================================================================
class App:
	"""Main App Class"""
#-----------------------------------------------------------------------------
	def __init__(self, master):
		"""Use a grid to layout the UI in rows and columns"""
		row_number = 0
		self.information_label = tk.Label(master, 
			text="Mortgage Calculator v. %s" % GUIVERSION).grid(row=row_number,
																columnspan=2,
																padx=10)
		self.help_button = tk.Button(master, 
			text="Help", command=self.help_button_pressed).grid(row=row_number, 
															column=2, 
															padx=10, 
															pady=10)
		row_number = 1
		self.principal_label = tk.Label(master, 
									text="Principal").grid(row=row_number, 
															sticky=tk.W, 
															padx=10)
		self.principal_entry = tk.Entry(master)
		self.principal_entry.grid(row=row_number, column=1)
		self.principal_button = tk.Button(master, 
			text="Calculate", 
			command=self.calc_principal_pressed).grid(
				row=row_number, 
				column=2, 
				padx=10)
		row_number = 2
		self.payment_label = tk.Label(master, 
			text="Monthly Payment").grid(row=row_number, sticky=tk.W, padx=10)
		self.payment_entry = tk.Entry(master)
		self.payment_entry.grid(row=row_number, column=1)
		self.payment_button = tk.Button(master, 
			text="Calculate", 
			command=self.calc_payment_pressed).grid(row=row_number, 
															column=2, 
															padx=10)
		row_number = 3
		self.number_of_payments_label = tk.Label(master, 
			text="Number of payments").grid(row=row_number, 
											sticky=tk.W, 
											padx=10)
		self.number_of_payments_entry = tk.Entry(master)
		self.number_of_payments_entry.grid(row=row_number, column=1)
		self.number_of_payments_button = tk.Button(master, 
			text="Calculate", 
			command=self.calc_number_of_payments_pressed). \
				grid(row=row_number, 
				  column=2, 
				  padx=10)
		row_number = 4
		self.interest_rate_label = tk.Label(master, 
			text="Interest rate % (ex. 4.25)").grid(row=row_number, 
									   sticky=tk.W, 
									   padx=10)
		self.interest_rate_entry = tk.Entry(master)
		self.interest_rate_entry.grid(row=row_number, column=1)
		self.interest_rate_button = tk.Button(master, 
			text="Calculate", 
			command=self.calc_interest_rate_pressed). \
				grid(row=row_number, 
					 column=2, 
					 padx=10)
		row_number = 5
		self.clear_all_button = tk.Button(master, 
			text="Clear All", 
			command=self.clear_all).grid(row=row_number, column=1)
		self.about_button = tk.Button(master, 
			text="About", command=self.display_about_box).grid(row=row_number, 
															 column=2, 
															 padx=10, 
															 pady=10)
		self.my_mortgage = mortgage.Mortgage()
#-----------------------------------------------------------------------------
	def help_button_pressed(self):
		"""Display Help"""
		tkMessageBox.showinfo("Mortgage Calculator Help", 
			"Fill in any 3 values, then click calculate next to the empty \
field, and the program will fill in the missing value. \
\n\nJust use numeric values, there is no need to put a dollar sign, or \
percent symbol.")		   
#-----------------------------------------------------------------------------
	def display_about_box(self):
		"""Display Versions and information"""
		tkMessageBox.showinfo ("About Mortgage Calculator", 
			"A Mortgage Calculator\nWritten by Joshua Grigonis \
			\nemail: josh.grigonis@gmail.com\n \
			\nMortgage GUI Version: %s\nMortgage Class Version: %s" 
			% (GUIVERSION, mortgage.Mortgage.version))	
#-----------------------------------------------------------------------------
	def clear_all(self):
		"""Clear all the mortgage data entry fields.
		
		Also, reset m to a new Mortgage.
	
		"""
		# Clear all Entry fields	
		self.principal_entry.delete(0, tk.END)
		self.payment_entry.delete(0, tk.END)
		self.number_of_payments_entry.delete(0, tk.END)
		self.interest_rate_entry.delete(0, tk.END)
		# Reset the mortgage
		self.my_mortgage = mortgage.Mortgage()
#-----------------------------------------------------------------------------
	def verify_interest_rate(self, interest_rate):
		"""Ask the user if the interest_rate is correct
		
		If the user indicates yes, then simply return the interest_rate.
		If the user indicates no, multiply it by 100 and return answer.
		I had to use a little hack to get the message to display % symbol.
		
		"""
		if 0.85 > interest_rate > 0:
			if not tkMessageBox.askyesno(title="Correct Interest Rate", 
					 message="Click yes, if you really mean %0.3f%s. \
					 \nClick No to change the interest rate to %0.3f%s." % 
					 (interest_rate, '%', interest_rate * 100, '%')):
				interest_rate = interest_rate * 100
		return interest_rate
#-----------------------------------------------------------------------------
	def calc_principal_pressed(self):
		"""Handle the Principal button being pressed
		
		The user has clicked the calculate principal button.
		Either fill in the principal, or beep if the necessary fields 
		are not filled in.
		Verify the interest_rate is what the user intended.
		
		"""
		try:
			interest_rate = convertTypes.makeFloat(
				self.interest_rate_entry.get(),
				requirePositive=True)
			interest_rate = self.verify_interest_rate(interest_rate)
		except ValueError:
			ROOT.bell()
			return
		self.interest_rate_entry.delete(0, tk.END)
		self.interest_rate_entry.insert(tk.INSERT, round(interest_rate, 4))
		try:
			self.my_mortgage = \
				mortgage.Mortgage(payment=self.payment_entry.get(), 
				numberOfPayments=self.number_of_payments_entry.get(), 
				interestRate=interest_rate)
		except ValueError:
			# Beep
			ROOT.bell()
			return
		try:
			principal = self.my_mortgage.calculatePrincipal()
		except mortgageExceptions.UnsetVariableError:
			ROOT.bell()
			return
		except:
			raise
		self.principal_entry.delete(0, tk.END)
		self.principal_entry.insert(tk.INSERT, round(principal, 2))
#-----------------------------------------------------------------------------
	def calc_payment_pressed(self):
		"""Handle the Calculate Payment button being pressed
		
		The user has clicked the calculate payment button.
		Either fill in the payment, or beep if the necessary 
		fields are not filled in.
		Verify the interest_rate is what the user intended.
		
		"""
		try:
			interest_rate = convertTypes.makeFloat(
				self.interest_rate_entry.get(),
				requirePositive=True)
			interest_rate = self.verify_interest_rate(interest_rate)
		except ValueError:
			ROOT.bell()
			return
		self.interest_rate_entry.delete(0, tk.END)
		self.interest_rate_entry.insert(tk.INSERT, round(interest_rate, 4))
		try:
			self.my_mortgage = \
				mortgage.Mortgage(principal=self.principal_entry.get(), 
				numberOfPayments=self.number_of_payments_entry.get(), 
				interestRate=interest_rate)
		except ValueError:
			# Beep
			ROOT.bell()
			return
		try:
			payment = self.my_mortgage.calculatePayment()
		except mortgageExceptions.UnsetVariableError:
			ROOT.bell()
			return
		except:
			raise
		self.payment_entry.delete(0, tk.END)
		self.payment_entry.insert(tk.INSERT, round(payment, 2))
#-----------------------------------------------------------------------------
	def calc_number_of_payments_pressed(self):
		"""Handle the Number of Payments button being pressed
		
		The user has clicked the calculate number of payments button.
		Either fill in the Number of Payments, or beep if the necessary 
		fields are not filled in.
		If the payment is too low, show an explanative message box.
		number_of_payments is rounded to 3 significant decimals.
		Verify the interest_rate is what the user intended.
		
		"""
		try:
			interest_rate = convertTypes.makeFloat(
				self.interest_rate_entry.get(),
				requirePositive=True)
			interest_rate = self.verify_interest_rate(interest_rate)
		except ValueError:
			ROOT.bell()
			return
		self.interest_rate_entry.delete(0, tk.END)
		self.interest_rate_entry.insert(tk.INSERT, round(interest_rate, 4))
		try:
			self.my_mortgage = \
				mortgage.Mortgage(principal=self.principal_entry.get(), 
				payment=self.payment_entry.get(), 
				interestRate=interest_rate)
		except ValueError:
			# Beep
			ROOT.bell()
			return
		try:
			number_of_payments = self.my_mortgage.calculateNumberOfPayments()
		except mortgageExceptions.PaymentTooLowError:
			ROOT.bell()
			tkMessageBox.showwarning("Payment Too Low Error", 
				"The payment is too low, or the principal and interest is \
too high.\n \
\nRaise the payments or lower the principal and/or interest.")
			return
		except mortgageExceptions.UnsetVariableError:
			ROOT.bell()
			return
		self.number_of_payments_entry.delete(0, tk.END)
		self.number_of_payments_entry.insert(tk.INSERT, 
										  round(number_of_payments, 3))
#-----------------------------------------------------------------------------
	def calc_interest_rate_pressed(self):
		"""Handle the Interest Rate button being pressed
		
		The user has clicked the calculate Interest Rate button.
		Either fill in the Interest Rate, or beep if the necessary 
		fields are not filled in.
		If the payment is too low, show an explanative message box.
		interest_rate is rounded to 3 significant desmials.
		
		"""
		try:
			self.my_mortgage = \
				mortgage.Mortgage(principal=self.principal_entry.get(), 
				numberOfPayments=self.number_of_payments_entry.get(), 
				payment=self.payment_entry.get())
		except ValueError:
			# Beep
			ROOT.bell()
			return
		try:
			interest_rate = self.my_mortgage.calculateInterestRate()
		except mortgageExceptions.PaymentTooLowError:
			ROOT.bell()
			tkMessageBox.showwarning("Payment Too Low Error", 
				"The payment, or the number of them is too low.\n \
\nRaise the payments, or number of payments, or \
lower the principal")
			return
		except mortgageExceptions.UnsetVariableError:
			ROOT.bell()
			return
		except:
			raise
		
		self.interest_rate_entry.delete(0, tk.END)
		self.interest_rate_entry.insert(tk.INSERT, round(interest_rate, 3))
		
#=============================================================================
if __name__ == '__main__':
	ROOT = tk.Tk()
	ROOT.title('Mortgage Calculator')
	APP = App(ROOT)
	ROOT.mainloop()